#import "NavigationEx.h"

//NavigationExの実装
@implementation NavigationEx

//アラートの表示
- (void)showAlert:(NSString*)title text:(NSString*)text {
    UIAlertView* alert=[[UIAlertView alloc]
        initWithTitle:title message:text delegate:nil 
        cancelButtonTitle:@"OK" otherButtonTitles:nil];
    [alert show];
}

//ファイルとフォルダ階層の定義(2)
- (id)init {
    self=[super init];
    if (self) {
        //フォルダ3-3
        NSMutableDictionary* folder3_3=[NSMutableDictionary dictionary];
        NSMutableArray*      files3_3 =[NSMutableArray array];
        [folder3_3 setObject:files3_3 forKey:@"フォルダ3-3"];
        [files3_3 addObject:@"ファイル3-3-1"];
        [files3_3 addObject:@"ファイル3-3-2"];
    
        //フォルダ1
        NSMutableDictionary* folder1=[NSMutableDictionary dictionary];
        NSMutableArray*      files1 =[NSMutableArray array];
        [folder1 setObject:files1 forKey:@"フォルダ1"];
        [files1 addObject:@"ファイル1-1"];
        [files1 addObject:@"ファイル1-2"];
        [files1 addObject:@"ファイル1-3"];

        //フォルダ2
        NSMutableDictionary* folder2=[NSMutableDictionary dictionary];
        NSMutableArray*      files2 =[NSMutableArray array];
        [folder2 setObject:files2 forKey:@"フォルダ2"];
        [files2 addObject:@"ファイル2-1"];
        [files2 addObject:@"ファイル2-2"];

        //フォルダ3
        NSMutableDictionary* folder3=[NSMutableDictionary dictionary];
        NSMutableArray*      files3 =[NSMutableArray array];
        [folder3 setObject:files3 forKey:@"フォルダ3"];
        [files3 addObject:@"ファイル3-1"];
        [files3 addObject:@"ファイル3-2"];
        [files3 addObject:folder3_3];

        //ホーム
        NSMutableDictionary* folder0=[NSMutableDictionary dictionary];
        NSMutableArray* files0=[NSMutableArray array];
        [folder0 setObject:files0 forKey:@"ホーム"];
        [files0 addObject:folder1];
        [files0 addObject:folder2];
        [files0 addObject:folder3];
        _folder=folder0;
    }
    return self;
}

//初期化
- (id)initWithFolder:(NSDictionary*)folder {
    self=[super init];
    if (self) {
        _folder=folder;
    }
    return self;
}

//ロード完了時に呼ばれる
- (void)viewDidLoad {
    [super viewDidLoad];
    
    //ナビゲーションバーのタイトルの指定(3)
    self.title=[[_folder allKeys] objectAtIndex:0];

    //ナビゲーションバーのボタンの追加(4)
    UIBarButtonItem* button=[[UIBarButtonItem alloc] 
        initWithTitle:@"ボタン" 
        style:UIBarButtonItemStyleBordered
        target:self 
        action:@selector(clickButton:)]; 
    self.navigationItem.rightBarButtonItem=button;

    //テーブルビューの生成(6)
    _tableView=[[UITableView alloc] init];
    [_tableView setFrame:self.view.frame];
    [_tableView setDelegate:self];                     
    [_tableView setDataSource:self];
    [self.view addSubview:_tableView];

    //ビューのサイズの自動調整
    _tableView.autoresizingMask=
        UIViewAutoresizingFlexibleRightMargin|
        UIViewAutoresizingFlexibleTopMargin|
        UIViewAutoresizingFlexibleLeftMargin|
        UIViewAutoresizingFlexibleBottomMargin|
        UIViewAutoresizingFlexibleWidth|
        UIViewAutoresizingFlexibleHeight;
}

//セルの高さの取得(7)
- (CGFloat)tableView:(UITableView*)tableView 
    heightForRowAtIndexPath:(NSIndexPath*)indexPath {
    return 50;
}

//セルのクリック時に呼ばれる(7)
- (void)tableView:(UITableView*)tableView 
    didSelectRowAtIndexPath:(NSIndexPath*)indexPath {
    NSString* key  =[[_folder allKeys] objectAtIndex:0];
    NSArray*  files=[_folder objectForKey:key];
    NSObject* obj  =[files objectAtIndex:indexPath.row];
    
    //フォルダ(1つ下の階層へ移動)(5)
    if ([[obj class] isSubclassOfClass:[NSDictionary class]]) {
        NavigationEx* viewCtl=[[NavigationEx alloc] 
            initWithFolder:(NSDictionary*)obj];
        [self.navigationController pushViewController:viewCtl animated:YES];    
    } 
    //ファイル
    else if ([[obj class] isSubclassOfClass:[NSString class]]) {
        [self showAlert:@"" text:(NSString*)obj];
    }

    //テーブルのセルの選択解除(9)
    [_tableView deselectRowAtIndexPath:
        [_tableView indexPathForSelectedRow] animated:YES];
}

//セルの数取得時に呼ばれる(8)
- (NSInteger)tableView:(UITableView*)tableView 
    numberOfRowsInSection:(NSInteger)section {
    NSString* key  =[[_folder allKeys] objectAtIndex:0];
    NSArray*  files=[_folder objectForKey:key];
    return files.count;
}

//セルの取得時に呼ばれる(8)
- (UITableViewCell*)tableView:(UITableView*)tableView 
    cellForRowAtIndexPath:(NSIndexPath*)indexPath {
    //テーブルのセルの生成(8)
    UITableViewCell* cell=[tableView 
        dequeueReusableCellWithIdentifier:@"table_cell"];
    if (cell==nil) {
        cell=[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault
            reuseIdentifier:@"table_cell"];
    }
    NSString* key  =[[_folder allKeys] objectAtIndex:0];
    NSArray*  files=[_folder objectForKey:key];
    NSObject* obj  =[files objectAtIndex:indexPath.row];

    //フォルダ
    if ([[obj class] isSubclassOfClass:[NSDictionary class]]) {
        [cell.textLabel setText:[[(NSDictionary*)obj allKeys] objectAtIndex:0]];
        cell.accessoryType=UITableViewCellAccessoryDisclosureIndicator;
    } 
    //ファイル
    else if ([[obj class] isSubclassOfClass:[NSString class]]) {
        [cell.textLabel setText:(NSString*)obj];
    }
    return cell;
}

//ボタンクリック時に呼ばれる
- (IBAction)clickButton:(UIButton*)sender {
    [self showAlert:@"" text:@"バーボタンを押した"];
}
@end
