#import "AccelerometerEx.h"
#define FILTERING_FACTOR 0.1

//AccelerometerExの実装
@implementation AccelerometerEx

//ラベルのリサイズ
- (void)resizeLabel:(UILabel*)label {
    [label setFrame:CGRectMake(
        label.frame.origin.x,
        label.frame.origin.y,
        9999,9999)];
    [label sizeToFit];
}

//ラベルの生成
- (UILabel*)makeLabel:(CGPoint)pos
    text:(NSString*)text font:(UIFont*)font {
    UILabel* label=[[UILabel alloc] init];
    [label setFrame:CGRectMake(pos.x,pos.y,9999,9999)];
    [label setText:text];
    [label setFont:font];
    [label setTextAlignment:NSTextAlignmentLeft];
    [label setLineBreakMode:NSLineBreakByWordWrapping];
    [label setNumberOfLines:0];
    [label sizeToFit];
    return label;
}

//モーション通知時の処理
- (void)updateMotion:(CMDeviceMotion*)motion {
    NSMutableString* str=[NSMutableString string];
    
    //端末の加速度の取得(2)
    CMAcceleration gravity=motion.gravity;
    [str appendString:@"AccelerometerEx\n"];
    
    //加速度にローパスフィルタをあてる(3)
    _aX=(gravity.x*FILTERING_FACTOR)+(_aX*(1.0-FILTERING_FACTOR));
    _aY=(gravity.y*FILTERING_FACTOR)+(_aY*(1.0-FILTERING_FACTOR));
    _aZ=(gravity.z*FILTERING_FACTOR)+(_aZ*(1.0-FILTERING_FACTOR));
    
    //加速度の更新
    [str appendFormat:@"X軸加速度:%+.2f\n",_aX];
    [str appendFormat:@"Y軸加速度:%+.2f\n",_aY];
    [str appendFormat:@"Z軸加速度:%+.2f\n",_aZ];
    [str appendString:@"\n"];
    
    
    if (_motionManager.gyroAvailable) {
        //端末の傾きの取得(4)
        CMAttitude* attitude=motion.attitude;
        
        //端末の傾きの更新
        [str appendFormat:@"X軸回転角度:%+.2f\n",attitude.pitch*180/M_PI];
        [str appendFormat:@"Y軸回転角度:%+.2f\n",attitude.yaw  *180/M_PI];
        [str appendFormat:@"Z軸回転角度:%+.2f\n",attitude.roll *180/M_PI];
        [str appendString:@"\n"];
    }
    
    //端末の向きの更新
    [str appendFormat:@"端末の向き:%@\n",_orientation];
    [str appendString:@"\n"];
    
    //ラベルの更新
    [_label setText:str];
    [self resizeLabel:_label];
}

//端末の向きの通知受信時の処理(6)
- (void)didRotate:(NSNotification*)notification {
    UIDeviceOrientation orientation=[[notification object] orientation];
    if (orientation==UIDeviceOrientationLandscapeLeft) {
        _orientation=@"横(左90度回転)";
    } else if (orientation==UIDeviceOrientationLandscapeRight) {
        _orientation=@"横(右90度回転)";
    } else if (orientation==UIDeviceOrientationPortraitUpsideDown) {
        _orientation=@"縦(上下逆)";
    } else if (orientation==UIDeviceOrientationPortrait) {
        _orientation=@"縦";
    } else if (orientation==UIDeviceOrientationFaceUp) {
        _orientation=@"画面が上向き";
    } else if (orientation==UIDeviceOrientationFaceDown) {
        _orientation=@"画面が下向き";
    }
}

//初期化
- (void)viewDidLoad {
    [super viewDidLoad];

    //ラベルの生成
    _label=[self makeLabel:CGPointMake(0,20)
        text:@"AccelerometerEx" 
        font:[UIFont systemFontOfSize:24]];
    [self.view addSubview:_label];

    //値の初期化
    _aX=0;
    _aY=0;
    _aZ=0;
    _orientation=@"";
    
    //センサー情報の通知の開始(1)
    _motionManager=[[CMMotionManager alloc] init];
    _motionManager.deviceMotionUpdateInterval=1.0/60.0;
    [_motionManager startDeviceMotionUpdatesToQueue:[NSOperationQueue currentQueue]
        withHandler:^(CMDeviceMotion* motion,NSError* error) {
            [self updateMotion:motion];
        }];

    //端末の向き通知の開始(5)
    [[UIDevice currentDevice]
        beginGeneratingDeviceOrientationNotifications];
    [[NSNotificationCenter defaultCenter] addObserver:self
        selector:@selector(didRotate:)
        name:UIDeviceOrientationDidChangeNotification
        object:nil];
}
@end
