#import "LocationEx.h"

//LocationExの実装
@implementation LocationEx

//アラートの表示
- (void)showAlert:(NSString*)title text:(NSString*)text {
    UIAlertView* alert=[[UIAlertView alloc]
        initWithTitle:title message:text delegate:nil 
        cancelButtonTitle:@"OK" otherButtonTitles:nil];
    [alert show];
}

//ラベルのリサイズ
- (void)resizeLabel:(UILabel*)label {
    [label setFrame:CGRectMake(
        label.frame.origin.x,
        label.frame.origin.y,
        9999,9999)];
    [label sizeToFit];
}

//ラベルの生成
- (UILabel*)makeLabel:(CGPoint)pos
    text:(NSString*)text font:(UIFont*)font {
    UILabel* label=[[UILabel alloc] init];
    [label setFrame:CGRectMake(pos.x,pos.y,9999,9999)];
    [label setText:text];
    [label setFont:font];
    [label setTextAlignment:NSTextAlignmentLeft];
    [label setLineBreakMode:NSLineBreakByWordWrapping];
    [label setNumberOfLines:0];
    [label sizeToFit];
    return label;
}

//初期化
- (void)viewDidLoad {
    [super viewDidLoad];
    
    //ラベルの生成
    _label=[self makeLabel:CGPointMake(0,20)
        text:@"LocationEx" 
        font:[UIFont systemFontOfSize:24]];
    [self.view addSubview:_label];    
    
    //情報
    _latitude =0.0f;
    _longitude=0.0f;
    _heading  =0.0f;
                
    //ロケーションマネージャーの生成(1)
    _locationManager=[[CLLocationManager alloc] init];
    _locationManager.delegate=self;
        
    //位置情報通知の開始(2)
    if ([CLLocationManager locationServicesEnabled]) {
        [_locationManager startUpdatingLocation];
    }        
        
    //方位情報通知の開始(3)
    if ([CLLocationManager headingAvailable]) {
        [_locationManager startUpdatingHeading];            
    }

    //タイマーの生成        
    [NSTimer scheduledTimerWithTimeInterval:0.1f
        target:self
        selector:@selector(onTick:)
        userInfo:nil
        repeats:YES];
}

//メモリ解放
- (void)dealloc {
    //位置情報通知の停止(2)
    [_locationManager stopUpdatingLocation];
    
    //方位情報通知の停止(3)
    [_locationManager stopUpdatingHeading];
    
    //メモリ解放
    _locationManager.delegate=nil;
}

//定期処理
- (void)onTick:(NSTimer*)timer {
    //ラベルの更新
    NSMutableString* str=[NSMutableString string];
    [str appendString:@"LocationEx\n"];
    [str appendFormat:@"緯度:%+f\n",_latitude];
    [str appendFormat:@"経度:%+f\n",_longitude];
    [str appendFormat:@"方位:%+3.2f\n",_heading];
    [_label setText:str];
    [self resizeLabel:_label];
}

//位置情報更新時に呼ばれる(4)
- (void)locationManager:(CLLocationManager*)manager 
    didUpdateToLocation:(CLLocation*)newLocation 
    fromLocation:(CLLocation*)oldLocation {
    //緯度・経度の取得
    CLLocationCoordinate2D coordinate=newLocation.coordinate;
    _latitude =coordinate.latitude;
    _longitude=coordinate.longitude;   
}

//位置情報取得失敗時に呼ばれる(4)
- (void)locationManager:(CLLocationManager*)manager 
    didFailWithError:(NSError*)error {
    if ([error code]==kCLErrorDenied) {
        [self showAlert:@"" text:@"位置情報取得は許可されていません"];
    } else {
        [self showAlert:@"" text:@"位置情報取得に失敗しました"];
    }
}

//方位情報更新時に呼ばれる(4)
- (void)locationManager:(CLLocationManager*)manager 
    didUpdateHeading:(CLHeading*)newHeading {
    //方位の取得
    _heading=newHeading.trueHeading;
}
@end
