#import "MyScene.h"

//シーンの実装
@implementation MyScene

//初期化時に呼ばれる
-(id)initWithSize:(CGSize)size {
    if (self=[super initWithSize:size]) {
        //シーンの設定
        self.backgroundColor=[SKColor blackColor];     //背景色
        self.scaleMode      =SKSceneScaleModeAspectFit;//スケールモード
        
        //バーの生成
        [self makeBar];
        
        //ボール追加アクションの追加(3)
        SKAction* makeBalls=[SKAction sequence: @[
            [SKAction performSelector:@selector(addBall) onTarget:self],
            [SKAction waitForDuration:0.1]
        ]];
        [self runAction:[SKAction repeatActionForever:makeBalls]];
    }
    return self;
}

//タッチ時に呼ばれる
- (void)touchesBegan:(NSSet*)touches withEvent:(UIEvent*)event {
}

//フレームの更新開始時に呼ばれる
- (void)update:(CFTimeInterval)currentTime {
}

//ノードのアクションの処理後に呼ばれる
- (void)didEvaluateActions {
}

//物理シミュレーションの処理後に呼ばれる
- (void)didSimulatePhysics {
    //ボールが画面下に落ちたら削除(4)
    [self enumerateChildNodesWithName:@"ball"
        usingBlock:^(SKNode* node, BOOL* stop) {
        if (node.position.y<0) [node removeFromParent];
    }];
}

//乱数の取得
- (int)rand:(int)num {
    return abs(arc4random())%num;
}

//ボールの追加
- (void)addBall {
    //スプライトの追加
    SKSpriteNode* ball=[SKSpriteNode spriteNodeWithImageNamed:@"ball.png"];
    ball.position = CGPointMake([self rand:self.size.width], self.size.height-50);
    ball.name = @"ball";
    [self addChild:ball];
    
    //衝突判定の指定(1)
    ball.physicsBody = [SKPhysicsBody bodyWithCircleOfRadius:10];
    ball.physicsBody.dynamic=YES;
}

//バーの追加
- (void)makeBar {
    //スプライトの追加
    SKSpriteNode* bar=[SKSpriteNode spriteNodeWithImageNamed:@"bar.png"];
    bar.position = CGPointMake(60, 100);
    [self addChild:bar];
    
    //左右移動アクションの追加(2)
    SKAction* action = [SKAction sequence:@[
        [SKAction waitForDuration:1.0],
        [SKAction moveByX:(self.size.width-120) y:0.0 duration:1.0],
        [SKAction waitForDuration:1.0],
        [SKAction moveByX:-(self.size.width-120) y:0 duration:1.0]]];
    [bar runAction: [SKAction repeatActionForever:action]];
    
    
    //衝突判定の指定(1)
    bar.physicsBody=[SKPhysicsBody bodyWithRectangleOfSize:CGSizeMake(120,12)];
    bar.physicsBody.dynamic=NO;
}
@end
