#import "PuzzleGame.h"

#define BTN_START 0

//PuzzleGameの実装
@implementation PuzzleGame

//乱数の取得
- (int)rand:(int)num {
    return abs(arc4random())%num;
}

//ラベルの生成
- (UILabel*)makeLabel:(CGRect)rect text:(NSString*)text font:(UIFont*)font {
    UILabel* label=[[UILabel alloc] init];
    [label setFrame:rect];
    [label setText:text];
    [label setFont:font];
    [label setTextColor:[UIColor whiteColor]];
    [label setTextAlignment:NSTextAlignmentCenter];
    [label setLineBreakMode:NSLineBreakByWordWrapping];
    [label setNumberOfLines:0];
    return label;
}

//イメージビューの生成
- (UIImageView*)makeImageView:(CGRect)rect image:(UIImage*)image {
    UIImageView* imageView=[[UIImageView alloc] init];
    [imageView setFrame:rect];
    [imageView setImage:image];
    return imageView;
}

//ピースイメージビューの生成
- (UIImageView*)makePieceImageView:(CGRect)rect index:(int)index
    picture:(UIImage*)picture piece:(UIImage*)piece {
    UIGraphicsBeginImageContextWithOptions(rect.size,NO,0);
    [picture drawInRect:CGRectMake(-75*(index%4),-75*(int)(index/4),300,300)];
    [piece drawInRect:CGRectMake(0,0,75,75)];
    UIImage* image=UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return [self makeImageView:rect image:image];
    
}

//イメージボタンの生成
- (UIButton*)makeButton:(CGRect)rect image:(UIImage*)image tag:(int)tag {
    UIButton* button=[UIButton buttonWithType:UIButtonTypeCustom];
    [button setFrame:rect];
    [button setImage:image forState:UIControlStateNormal];
    [button setTag:tag];
    [button addTarget:self action:@selector(clickButton:)
    forControlEvents:UIControlEventTouchUpInside];
    return button;
}

//初期化
- (void)viewDidLoad {
    [super viewDidLoad];
    
    //ゲーム画面のXY座標とスケールの指定(1)
    CGSize screenSize=[[UIScreen mainScreen] bounds].size;
    float vx=(screenSize.width-360)/2;
    float vy=(screenSize.height-640)/2;
    float scale=screenSize.width/360;
    _gameView=[[UIView alloc] init];
    _gameView.frame=CGRectMake(vx,vy,360,640);
    _gameView.transform=CGAffineTransformMakeScale(scale,scale);
    [self.view addSubview:_gameView];
    
    //背景の生成
    UIImageView* bg=[self makeImageView:
        CGRectMake(0,0,360,640)
        image:[UIImage imageNamed:@"bg.png"]];
    [_gameView addSubview:bg];
    
    //絵の背景の生成
    UIImageView* picturebg=[self makeImageView:
        CGRectMake(29,179,302,302)
        image:[UIImage imageNamed:@"picturebg.png"]];
    [_gameView addSubview:picturebg];

    //タイトルラベルの生成
    _titleLabel=[self makeLabel:
        CGRectMake(0,90,360,70)
        text:@"Moon Puzzle"
        font:[UIFont systemFontOfSize:48]];
    [_gameView addSubview:_titleLabel];
    
    
    //絵のビットマップの取得
    UIImage* picture=[UIImage imageNamed:@"picture.png"];
    UIImage* piece=[UIImage imageNamed:@"piece.png"];
    for (int i=0;i<16;i++) {
        _piece[i]=[self makePieceImageView:
            CGRectMake(30+(i%4)*75,180+(int)(i/4)*75,75,75)
            index:i picture:picture piece:piece];
        [_gameView addSubview:_piece[i]];
        _data[i]=i;
    }
    
    //スタートボタンの生成
    _startButton=[self makeButton:CGRectMake(124,455,114,114)
        image:[UIImage imageNamed:@"start.png"] tag:BTN_START];
    [_gameView addSubview:_startButton];
}

//ボタンクリック時に呼ばれる
- (IBAction)clickButton:(UIButton*)sender {
    if (sender.tag==BTN_START) {
        //シャッフルの実行(3)
        _shuffle=20;
        while (_shuffle>0) {
            if ([self movePiece_x:[self rand:4] y:[self rand:4]]) _shuffle--;
        }
        for (int i=0; i<16; i++) {
            int dx=30+75*(i%4);
            int dy=180+75*(int)(i/4);
            _piece[_data[i]].frame=CGRectMake(dx,dy,75,75);
        }
        
        //ゲーム開始
        _titleLabel.text=@"Moon Puzzle";
        _piece[15].alpha=0;
        _startButton.alpha=0;
    }
}

//タッチ開始イベントの処理
- (void)touchesBegan:(NSSet*)touches withEvent:(UIEvent*)event {
    if (_startButton.alpha!=0) return;
    
    //タッチ位置の取得
    CGPoint pos=[[[touches allObjects] objectAtIndex:0]
        locationInView:_gameView];
    if (30<pos.x && pos.x<330 && 180<pos.y && pos.y<480) {
        int tx=(int)((pos.x-(30))/75);//(4)
        int ty=(int)((pos.y-(180))/75);//(4)
        [self movePiece_x:tx y:ty];
    }
}

//ピースの移動(4)
- (BOOL)movePiece_x:(int)tx y:(int)ty {
    //空きマスの行番号と列番号の計算(5)
    int fx=0;
    int fy=0;
    for (int i=0; i<16; i++) {
        if (_data[i] == 15) {
            fx=i%4;
            fy=(int)(i/4);
            break;
        }
    }
    if ((fx==tx && fy==ty) ||
        (fx!=tx && fy!=ty)) return NO;
    
    //ピースを上にシフト(6)
    if (fx==tx && fy<ty) {
        for (int i=fy; i<ty; i++) {
            _data[fx+i*4]=_data[fx+i*4+4];
        }
        _data[tx+ty*4]=15;
    }
    //ピースを下にシフト(6)
    else if (fx==tx && fy>ty) {
        for (int i=fy; i>ty; i--) {
            _data[fx+i*4]=_data[fx+i*4-4];
        }
        _data[tx+ty*4]=15;
    }
    //ピースを左にシフト(6)
    else if (fy==ty && fx<tx) {
        for (int i=fx; i<tx; i++) {
            _data[i+fy*4]=_data[i+fy*4+1];
        }
        _data[tx+ty*4]=15;
    }
    //ピースを右にシフト(6)
    else if (fy==ty && fx>tx) {
        for (int i=fx; i>tx; i--) {
            _data[i+fy*4]=_data[i+fy*4-1];
        }
        _data[tx+ty*4]=15;
    }
    if (_shuffle>0) return YES;
    
    //ピースの移動アニメとクリアチェック(7)
    int clearCheck=0;
    for (int i=0; i<16; i++) {
        int dx=30+75*(i%4);
        int dy=180+75*(int)(i/4);
        
        //ピースの移動アニメ
        if (_data[i]!=15) {
            [UIView beginAnimations:@"anime0" context:NULL];
            [UIView setAnimationDuration:0.3f];
            _piece[_data[i]].frame=CGRectMake(dx,dy,75,75);
            [UIView commitAnimations];
        } else {
            _piece[_data[i]].frame=CGRectMake(dx,dy,75,75);
        }
        
        //クリアチェック
        if (_data[i]==i) clearCheck++;
    }
    
    //ゲームのクリア判定(8)
    if (clearCheck==16) {
        _titleLabel.text=@"Clear!";
        _startButton.alpha=100;
        
        //ピースの出現アニメ
        [UIView beginAnimations:@"anime1" context:NULL];
        [UIView setAnimationDuration:0.6f];
        _piece[15].alpha=100;
        [UIView commitAnimations];
    }
    return YES;
}
@end
