#import "MyScene.h"

//シーンの実装
@implementation MyScene

//乱数の取得
- (int)rand:(int)num {
    return abs(arc4random())%num;
}

//2点の距離の計算(8)
- (int)distance_x0:(int)x0 y0:(int)y0 x1:(int)x1 y1:(int)y1 {
    return sqrt((x0-x1)*(x0-x1)+(y0-y1)*(y0-y1));
}

//初期化時に呼ばれる
-(id)initWithSize:(CGSize)size {
    if (self=[super initWithSize:size]) {
        //スケールモードの指定
        self.scaleMode=SKSceneScaleModeAspectFill;//(1)
        
        //シーンの設定
        _scene=SCENE_TITLE;
        
        //画面サイズの取得
        CGSize screenSize=[[UIScreen mainScreen] bounds].size;
        float scale = 360/screenSize.width;
        
        //背景の生成
        SKSpriteNode* bg=[SKSpriteNode spriteNodeWithImageNamed:@"bg.png"];
        bg.position=CGPointMake(180,320);
        [self addChild:bg];
        
        //ビルの生成
        _buildingTex[0]=[SKTexture textureWithImageNamed:@"building0.png"];//(3)
        _buildingTex[1]=[SKTexture textureWithImageNamed:@"building1.png"];//(3)
        _building=[[SKSpriteNode alloc]
            initWithColor:[UIColor whiteColor] size:CGSizeMake(360,140)];
        _building.texture=_buildingTex[0];//(3)
        _building.position=CGPointMake(180,70);
        [self addChild:_building];
        
        //隕石レイヤーの生成(4)
        _meteoLayer=[[SKSpriteNode alloc]
            initWithColor:[SKColor clearColor] size:CGSizeMake(360,640)];
        _meteoLayer.anchorPoint=CGPointMake(0,0);
        [self addChild:_meteoLayer];
        
        //弾レイヤーの生成(4)
        _shotLayer=[[SKSpriteNode alloc]
             initWithColor:[SKColor clearColor] size:CGSizeMake(360,640)];
        _shotLayer.anchorPoint=CGPointMake(0,0);
        [self addChild:_shotLayer];
        
        //宇宙船の生成
        _ship=[SKSpriteNode spriteNodeWithImageNamed:@"ship.png"];
        _ship.position=CGPointMake(180,180);
        [self addChild:_ship];
        
        //スコアラベルの生成
        _scoreLabel=[SKLabelNode labelNodeWithFontNamed:@"ArialMT"];
        _scoreLabel.text     =@"SCORE:00000000";
        _scoreLabel.fontSize =20;
        _scoreLabel.fontColor=[SKColor whiteColor];
        _scoreLabel.position =CGPointMake(265,
            (640-screenSize.height*scale)/2+screenSize.height*scale-40);
        [self addChild:_scoreLabel];
        
        //タイトルの生成
        _title=[SKSpriteNode spriteNodeWithImageNamed:@"title.png"];
        _title.position=CGPointMake(180,360);

        //ゲームオーバーの生成
        _gameover=[SKSpriteNode spriteNodeWithImageNamed:@"gameover.png"];
        _gameover.position=CGPointMake(180,360);
        
        //シーンをタイトルに遷移
        [self initScene:SCENE_TITLE];
    }
    return self;
}

//シーンの初期化(2)
- (void)initScene:(int)scene {
    _scene=scene;
    
    //文字の削除
    [_title removeFromParent];
    [_gameover removeFromParent];
    
    //アクションの削除
    [self removeAllActions];
    for (int i=0; i<_shotLayer.children.count; i++) {
        SKNode* shot=[_shotLayer.children objectAtIndex:i];
        [shot removeAllActions];
    }
    for (int i=0; i<_meteoLayer.children.count; i++) {
        SKNode* meteo=[_meteoLayer.children objectAtIndex:i];
        [meteo removeAllActions];
    }
    
    //タイトル
    if (_scene==SCENE_TITLE) {
        //タイトルの表示
        [self addChild:_title];
        
        //宇宙船の位置
        _ship.position=CGPointMake(180,180);
        
        //ビルの表示
        _building.texture=_buildingTex[0];//(3)
        
        //スコアのリセット
        _scoreLabel.text=@"SCORE:00000000";
        _score=0;
        
        //削除
        [_meteoLayer removeAllChildren];
        [_shotLayer removeAllChildren];
    }
    //プレイ
    else if (_scene==SCENE_PLAY) {
        //隕石追加アクションの追加(5)
        SKAction* makeMeteo=[SKAction sequence: @[
            [SKAction performSelector:@selector(addMeteo) onTarget:self],
            [SKAction waitForDuration:1.0]
        ]];
        [self runAction:[SKAction repeatActionForever:makeMeteo]];
    }
    //ゲームオーバー
    else if (_scene==SCENE_GAMEOVER) {
        //ゲームオーバーの表示
        [self addChild:_gameover];
        
        //ビルの表示
        _building.texture=_buildingTex[1];//(3)
        
        //ゲームオーバー時間の指定
        _gameoverDate=[NSDate date];
    }
}

//タッチ時に呼ばれる
- (void)touchesBegan:(NSSet*)touches withEvent:(UIEvent*)event {
    //タイトル
    if (_scene==SCENE_TITLE) {
        [self initScene:SCENE_PLAY];
    }
    //プレイ
    else if (_scene==SCENE_PLAY) {
        //弾の追加
        [self addShot];
        
        //宇宙船移動アクションの追加
        CGPoint pos=[[[touches allObjects] objectAtIndex:0] locationInNode:self];
        [_ship runAction:[SKAction moveToX:pos.x duration:0.3f]];
    }
    //ゲームオーバー
    else if (_scene==SCENE_GAMEOVER) {
        //ゲームオーバー後1秒以上
        if (_gameoverDate!=nil &&
            [[NSDate date] timeIntervalSinceDate:_gameoverDate]>1) {
            [self initScene:SCENE_TITLE];
        }
    }
}

//タッチ移動時に呼ばれる
- (void)touchesMoved:(NSSet*)touches withEvent:(UIEvent*)event {
    //プレイ
    if (_scene==SCENE_PLAY) {
        //宇宙船移動アクションの追加
        CGPoint pos=[[[touches allObjects] objectAtIndex:0] locationInNode:self];
        [_ship runAction:[SKAction moveToX:pos.x duration:0.1f]];
    }
}

//フレームの更新開始時に呼ばれる
- (void)update:(CFTimeInterval)currentTime {
}

//ノードのアクションの処理後に呼ばれる
- (void)didEvaluateActions {
    //プレイ
    if (_scene==SCENE_PLAY) {
        //弾の定期処理(7)
        for (int i=0; i<_shotLayer.children.count; i++) {
            SKNode* shot=[_shotLayer.children objectAtIndex:i];
            //画面上端まで移動した時
            if (shot.position.y>=640) {
                [shot removeFromParent];
            }
            //弾と隕石が衝突した時
            else {
                for (int i=0; i<_meteoLayer.children.count; i++) {
                    SKNode* meteo=[_meteoLayer.children objectAtIndex:i];
                    if ([self distance_x0:shot.position.x y0:shot.position.y
                        x1:meteo.position.x y1:meteo.position.y]<20) {
                        [shot removeFromParent];
                        [meteo removeFromParent];
                        _score+=30;
                        _scoreLabel.text=[NSString stringWithFormat:
                            @"SCORE:%08d",_score];
                        [self addBom:meteo];
                    }
                }
            }
        }
        //隕石の定期処理(7)
        for (int i=0; i<_meteoLayer.children.count; i++) {
            SKNode* meteo=[_meteoLayer.children objectAtIndex:i];
            //画面下端まで移動した時
            if (meteo.position.y<=0) {
                [self initScene:SCENE_GAMEOVER];
            }
        }
    }
}

//物理シミュレーションの処理後に呼ばれる
- (void)didSimulatePhysics {
}

//隕石の追加(6)
- (void)addMeteo {
    SKSpriteNode* meteo=[SKSpriteNode spriteNodeWithImageNamed:@"meteo.png"];
    meteo.position=CGPointMake(20+[self rand:320],640);
    [_meteoLayer addChild:meteo];
    [meteo runAction:[SKAction moveToY:0 duration:7]];
}

//弾の追加(6)
- (void)addShot {
    SKSpriteNode* shot=[SKSpriteNode spriteNodeWithImageNamed:@"shot.png"];
    shot.position=CGPointMake(_ship.position.x,_ship.position.y);
    [_shotLayer addChild:shot];
    [shot runAction:[SKAction moveToY:640 duration:3]];
}

//爆発の追加(6)
- (void)addBom:(SKNode*)node {
    SKSpriteNode* bom=[SKSpriteNode spriteNodeWithImageNamed:@"bom.png"];
    bom.position=CGPointMake(node.position.x,node.position.y);
    [_meteoLayer addChild:bom];
    [bom runAction:[SKAction sequence:@[
        [SKAction waitForDuration:0.1],
        [SKAction removeFromParent]]]];
}
@end
